@@ -10,8 +10,8 @@
 #' STAMP computes the Matrix Profile and Profile Index in such manner that it can be stopped before
 #' its complete calculation and return the best so far results allowing ultra-fast approximate
 #' solutions. `verbose` changes how much information is printed by this function; `0` means nothing,
-#' `1` means text, `2` means text and sound. `exclusion_zone` is used to avoid  trivial matches; if
-#' a query data is provided (join similarity), this parameter is ignored.
+#' `1` means text, `2` adds the progress bar, `3` adds the finish sound. `exclusion_zone` is used to
+#' avoid  trivial matches; if a query data is provided (join similarity), this parameter is ignored.
 #'
 #' @param ... a `matrix` or a `vector`. If a second time series is supplied it will be a join matrix
 #'   profile.
@@ -132,18 +132,19 @@ stamp <- function(..., window_size, exclusion_zone = 1 / 2, verbose = 2, s_size
 
   j <- 1
   ssize <- min(s_size, num_queries)
-  order <- sample(1:num_queries, size = ssize)
+  order <- 1:num_queries
+  order <- sample(order, size = ssize)
 
   tictac <- Sys.time()
 
-  if (verbose > 0) {
-    pb <- utils::txtProgressBar(min = 1, max = ssize, style = 3, width = 80)
+  if (verbose > 1) {
+    pb <- progress::progress_bar$new(
+      format = "STAMP [:bar] :percent at :tick_rate it/s, elapsed: :elapsed, eta: :eta",
+      clear = FALSE, total = ssize, width = 80
+    )
   }
 
-  if (verbose > 0) {
-    on.exit(close(pb))
-  }
-  if (verbose > 1) {
+  if (verbose > 2) {
     on.exit(beep(sounds[[1]]), TRUE)
   }
   # anytime must return the result always
@@ -176,12 +177,14 @@ stamp <- function(..., window_size, exclusion_zone = 1 / 2, verbose = 2, s_size
       exc_st <- max(1, i - exclusion_zone)
       exc_ed <- min(matrix_profile_size, i + exclusion_zone)
       distance_profile[exc_st:exc_ed] <- Inf
-      distance_profile[pre$data_sd < vars()$eps] <- Inf
-      if (skip_location[i] || any(pre$query_sd[i] < vars()$eps)) {
-        distance_profile[] <- Inf
-      }
     }
 
+    distance_profile[pre$data_sd < vars()$eps] <- Inf
+    if (skip_location[i] || any(pre$query_sd[i] < vars()$eps)) {
+      distance_profile[] <- Inf
+    }
+    distance_profile[skip_location] <- Inf
+
     # anytime version
     if (length(args) == 1) {
       # no RMP and LMP for joins
@@ -203,15 +206,15 @@ stamp <- function(..., window_size, exclusion_zone = 1 / 2, verbose = 2, s_size
     matrix_profile[ind] <- distance_profile[ind]
     profile_index[which(ind)] <- i
 
-    if (verbose > 0) {
-      utils::setTxtProgressBar(pb, j)
+    if (verbose > 1) {
+      pb$tick()
     }
   }
 
   tictac <- Sys.time() - tictac
 
   if (verbose > 0) {
-    message(sprintf("\nFinished in %.2f %s", tictac, units(tictac)))
+    message(sprintf("Finished in %.2f %s", tictac, units(tictac)))
   }
 
   # return() is at on.exit() function
